@@ -2,10 +2,15 @@
 Tests for api package.
 """
 
+import pytest
+
+import base64
+
 from django.core.urlresolvers import reverse
 
-from nsupdate.main.dnstools import query_ns
+from nsupdate.main.dnstools import query_ns, FQDN
 from nsupdate.main.models import Domain
+from nsupdate.api.views import basic_authenticate
 
 from conftest import TESTDOMAIN, TEST_HOST, TEST_HOST2, TEST_SECRET, TEST_SECRET2
 
@@ -13,7 +18,7 @@ USERNAME = 'test'
 PASSWORD = 'pass'
 
 BASEDOMAIN = "nsupdate.info"
-HOSTNAME = 'nsupdate-ddns-client-unittest.' + BASEDOMAIN
+TEST_HOST_OTHER = FQDN('nsupdate-ddns-client-unittest', BASEDOMAIN)
 
 
 def test_myip(client):
@@ -29,8 +34,25 @@ def test_nic_update_noauth(client):
 
 
 def make_basic_auth_header(username, password):
-    import base64
-    return b'Basic ' + base64.b64encode((username + ':' + password).encode('utf-8'))
+    """
+    create a basic authentication header
+
+    :param username: user name [unicode on py2, str on py3]
+    :param password: password [unicode on py2, str on py3]
+    :return: basic auth header [str on py2, str on py3]
+    """
+    # note: the coding dance in the next lines is to make sure we get str type
+    # on python 2 as well as on python 3 as str is the type we get in the auth
+    # object when practically running with a real web server.
+    user_pass = u'%s:%s' % (username, password)
+    return 'Basic ' + str(base64.b64encode(user_pass.encode('utf-8')).decode('ascii'))
+
+
+def test_basic_auth():
+    user_pass = "username", "secret"
+    h = make_basic_auth_header(*user_pass)
+    assert isinstance(h, str)  # must be str on py2, must be str on py3!
+    assert basic_authenticate(h) == user_pass
 
 
 def test_nic_update_badauth(client):
@@ -70,6 +92,20 @@ def test_nic_update_authorized_not_fqdn_username(client):
     assert response.content == b'notfqdn'
 
 
+def test_nic_update_authorized_invalid_ip1(client):
+    response = client.get(reverse('nic_update') + '?myip=1234',
+                          HTTP_AUTHORIZATION=make_basic_auth_header(TEST_HOST, TEST_SECRET))
+    assert response.status_code == 200
+    assert response.content == b'dnserr'
+
+
+def test_nic_update_authorized_invalid_ip2(client):
+    response = client.get(reverse('nic_update') + '?myip=%C3%A4%C3%BC%C3%B6',
+                          HTTP_AUTHORIZATION=make_basic_auth_header(TEST_HOST, TEST_SECRET))
+    assert response.status_code == 200
+    assert response.content == b'dnserr'
+
+
 def test_nic_update_authorized(client):
     response = client.get(reverse('nic_update'),
                           HTTP_AUTHORIZATION=make_basic_auth_header(TEST_HOST, TEST_SECRET))
@@ -112,6 +148,7 @@ def test_nic_update_authorized_myip(client):
     assert response.content == b'nochg 1.2.3.4'
 
 
+@pytest.mark.requires_sequential
 def test_nic_update_authorized_update_other_services(client):
     response = client.get(reverse('nic_update') + '?myip=4.3.2.1',
                           HTTP_AUTHORIZATION=make_basic_auth_header(TEST_HOST, TEST_SECRET))
@@ -123,15 +160,17 @@ def test_nic_update_authorized_update_other_services(client):
     assert response.status_code == 200
     # must be good (was different IP)
     assert response.content == b'good 1.2.3.4'
+    # XXX test below can not run in parallel (like on travis-ci.org) if updating same
+    # "other service" target host
     # now check if it updated the other service also:
-    assert query_ns(HOSTNAME, 'A') == '1.2.3.4'
+    assert query_ns(TEST_HOST_OTHER, 'A') == '1.2.3.4'
     response = client.get(reverse('nic_update') + '?myip=2.3.4.5',
                           HTTP_AUTHORIZATION=make_basic_auth_header(TEST_HOST, TEST_SECRET))
     assert response.status_code == 200
     # must be good (was different IP)
     assert response.content == b'good 2.3.4.5'
     # now check if it updated the other service also:
-    assert query_ns(HOSTNAME, 'A') == '2.3.4.5'
+    assert query_ns(TEST_HOST_OTHER, 'A') == '2.3.4.5'
 
 
 def test_nic_update_authorized_badagent(client, settings):
@@ -179,6 +218,20 @@ def test_nic_update_session_foreign_host(client):
     assert response.content == b'nohost'
 
 
+def test_nic_delete_authorized_invalid_ip1(client):
+    response = client.get(reverse('nic_delete') + '?myip=1234',
+                          HTTP_AUTHORIZATION=make_basic_auth_header(TEST_HOST, TEST_SECRET))
+    assert response.status_code == 200
+    assert response.content == b'dnserr'
+
+
+def test_nic_delete_authorized_invalid_ip2(client):
+    response = client.get(reverse('nic_delete') + '?myip=%C3%A4%C3%BC%C3%B6',
+                          HTTP_AUTHORIZATION=make_basic_auth_header(TEST_HOST, TEST_SECRET))
+    assert response.status_code == 200
+    assert response.content == b'dnserr'
+
+
 def test_nic_delete_authorized(client):
     response = client.get(reverse('nic_update') + '?myip=%s' % ('1.2.3.4', ),
                           HTTP_AUTHORIZATION=make_basic_auth_header(TEST_HOST, TEST_SECRET))
