@@ -9,17 +9,19 @@ from .models import Host, Domain, BlacklistedDomain, ServiceUpdater, ServiceUpda
 
 class DomainAdmin(admin.ModelAdmin):
     list_display = ("domain", "public", "available", "created_by")
-    list_filter = ("public", "available", "created_by")
+    list_filter = ("created", "public", "available")
+    search_fields = ("domain", "created_by__username", "created_by__email")
 
 
 class HostAdmin(admin.ModelAdmin):
     list_display = ("subdomain", "domain", "created_by", "client_faults", "abuse", "abuse_blocked")
-    list_filter = ("abuse", "abuse_blocked", "domain", "created_by")
+    list_filter = ("created", "abuse", "abuse_blocked", "domain")
+    search_fields = ("subdomain", "created_by__username", "created_by__email")
 
 
 class BlacklistedDomainAdmin(admin.ModelAdmin):
     list_display = ("domain", "created_by")
-    list_filter = ("created_by", "created")
+    list_filter = ("created", )
 
 
 admin.site.register(BlacklistedDomain, BlacklistedDomainAdmin)
