@@ -44,7 +44,6 @@ class GenerateSecretView(UpdateView):
         context['nav_overview'] = True
         # generate secret, store it hashed and return the plain secret for the context
         context['update_secret'] = self.object.generate_secret()
-        context['hosts'] = Host.objects.filter(created_by=self.request.user)
         messages.add_message(self.request, messages.SUCCESS, 'Host secret created.')
         return context
 
@@ -65,9 +64,8 @@ class GenerateNSSecretView(UpdateView):
 
     def get_context_data(self, *args, **kwargs):
         context = super(GenerateNSSecretView, self).get_context_data(*args, **kwargs)
-        context['nav_domain_overview'] = True
+        context['nav_overview'] = True
         context['shared_secret'] = self.object.generate_ns_secret()
-        context['domains'] = Domain.objects.filter(created_by=self.request.user)
         messages.add_message(self.request, messages.SUCCESS, 'Nameserver shared secret created.')
         return context
 
@@ -173,20 +171,38 @@ class JsUpdateView(TemplateView):
         return context
 
 
-class OverviewView(CreateView):
-    model = Host
+class OverviewView(TemplateView):
     template_name = "main/overview.html"
-    form_class = CreateHostForm
 
     @method_decorator(login_required)
     def dispatch(self, *args, **kwargs):
         return super(OverviewView, self).dispatch(*args, **kwargs)
 
+    def get_context_data(self, *args, **kwargs):
+        context = super(OverviewView, self).get_context_data(*args, **kwargs)
+        context['nav_overview'] = True
+        context['hosts'] = Host.objects.filter(created_by=self.request.user)
+        context['your_domains'] = Domain.objects.filter(
+            created_by=self.request.user)
+        context['public_domains'] = Domain.objects.filter(
+            public=True).exclude(created_by=self.request.user)
+        return context
+
+
+class AddHostView(CreateView):
+    template_name = "main/host_add.html"
+    model = Host
+    form_class = CreateHostForm
+
+    @method_decorator(login_required)
+    def dispatch(self, *args, **kwargs):
+        return super(AddHostView, self).dispatch(*args, **kwargs)
+
     def get_success_url(self):
         return reverse('generate_secret_view', args=(self.object.pk,))
 
     def get_form(self, form_class):
-        form = super(OverviewView, self).get_form(form_class)
+        form = super(AddHostView, self).get_form(form_class)
         form.fields['domain'].queryset = Domain.objects.filter(
             Q(created_by=self.request.user) | Q(public=True))
         return form
@@ -194,7 +210,7 @@ class OverviewView(CreateView):
     def form_valid(self, form):
         self.object = form.save(commit=False)
         try:
-            dnstools.add(self.object.get_fqdn(), self.request.META['REMOTE_ADDR'], origin=self.object.domain.domain)
+            dnstools.add(self.object.get_fqdn(), self.request.META['REMOTE_ADDR'])
         except dnstools.Timeout:
             success, level, msg = False, messages.ERROR, 'Timeout - communicating to name server failed.'
         except dnstools.NameServerNotAvailable:
@@ -217,9 +233,8 @@ class OverviewView(CreateView):
         return HttpResponseRedirect(url)
 
     def get_context_data(self, *args, **kwargs):
-        context = super(OverviewView, self).get_context_data(*args, **kwargs)
+        context = super(AddHostView, self).get_context_data(*args, **kwargs)
         context['nav_overview'] = True
-        context['hosts'] = Host.objects.filter(created_by=self.request.user)
         return context
 
 
@@ -251,7 +266,6 @@ class HostView(UpdateView):
         context = super(HostView, self).get_context_data(*args, **kwargs)
         context['nav_overview'] = True
         context['remote_addr'] = self.request.META['REMOTE_ADDR']
-        context['hosts'] = Host.objects.filter(created_by=self.request.user)
         return context
 
 
@@ -265,7 +279,9 @@ class DeleteHostView(DeleteView):
 
     def get_object(self, *args, **kwargs):
         obj = super(DeleteHostView, self).get_object(*args, **kwargs)
-        if obj.created_by != self.request.user:
+        if (obj.created_by != self.request.user or obj.abuse_blocked):
+            # disallow deletion if abuse_blocked is set, otherwise the
+            # abuser can just delete and recreate the host
             raise PermissionDenied()  # or Http404
         return obj
 
@@ -275,18 +291,17 @@ class DeleteHostView(DeleteView):
     def get_context_data(self, *args, **kwargs):
         context = super(DeleteHostView, self).get_context_data(*args, **kwargs)
         context['nav_overview'] = True
-        context['hosts'] = Host.objects.filter(created_by=self.request.user)
         return context
 
 
-class DomainOverviewView(CreateView):
+class AddDomainView(CreateView):
+    template_name = "main/domain_add.html"
     model = Domain
-    template_name = "main/domain_overview.html"
     form_class = CreateDomainForm
 
     @method_decorator(login_required)
     def dispatch(self, *args, **kwargs):
-        return super(DomainOverviewView, self).dispatch(*args, **kwargs)
+        return super(AddDomainView, self).dispatch(*args, **kwargs)
 
     def get_success_url(self):
         return reverse('generate_ns_secret_view', args=(self.object.pk,))
@@ -299,13 +314,8 @@ class DomainOverviewView(CreateView):
         return HttpResponseRedirect(self.get_success_url())
 
     def get_context_data(self, *args, **kwargs):
-        context = super(
-            DomainOverviewView, self).get_context_data(*args, **kwargs)
-        context['nav_domain_overview'] = True
-        context['your_domains'] = Domain.objects.filter(
-            created_by=self.request.user)
-        context['public_domains'] = Domain.objects.filter(
-            public=True).exclude(created_by=self.request.user)
+        context = super(AddDomainView, self).get_context_data(*args, **kwargs)
+        context['nav_overview'] = True
         return context
 
 
@@ -319,7 +329,7 @@ class DomainView(UpdateView):
         return super(DomainView, self).dispatch(*args, **kwargs)
 
     def get_success_url(self):
-        return reverse('domain_overview')
+        return reverse('overview')
 
     def form_valid(self, form):
         self.object = form.save(commit=False)
@@ -335,8 +345,7 @@ class DomainView(UpdateView):
 
     def get_context_data(self, *args, **kwargs):
         context = super(DomainView, self).get_context_data(*args, **kwargs)
-        context['nav_domain_overview'] = True
-        context['domains'] = Domain.objects.filter(created_by=self.request.user)
+        context['nav_overview'] = True
         return context
 
 
@@ -355,12 +364,11 @@ class DeleteDomainView(DeleteView):
         return obj
 
     def get_success_url(self):
-        return reverse('domain_overview')
+        return reverse('overview')
 
     def get_context_data(self, *args, **kwargs):
         context = super(DeleteDomainView, self).get_context_data(*args, **kwargs)
-        context['nav_domain_overview'] = True
-        context['domains'] = Domain.objects.filter(created_by=self.request.user)
+        context['nav_overview'] = True
         return context
 
 
@@ -469,7 +477,6 @@ Disallow: /update/
 Disallow: /host/
 Disallow: /overview/
 Disallow: /domain/
-Disallow: /domain_overview/
 Disallow: /updater_hostconfig/
 Disallow: /updater_hostconfig_overview/
 """
